﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.Globalization;
using AccesDB;
using NegociEntitats;

namespace Negoci
{
    public class configuracioCN
    {
        // Establim la connexió amb les classes que modelitzen la BD...
        DataClassesBibliotecaDataContext dc = new DataClassesBibliotecaDataContext();


        // ***********************************************************************************************
        //  Buscar la configuració 
        public configuracioEntity buscarConfiguracio()
        {
            try
            {
                var query = (from d in dc.configuracio
                             where d.id_config.Equals(1)
                             select d).Single();

                configuracioEntity obj = new configuracioEntity();
                obj.id_config = query.id_config;
                obj.nomBiblioteca = query.nomBiblioteca;
                obj.adrecaBiblioteca = query.adrecaBiblioteca;
                obj.cpBiblioteca = query.cpBiblioteca;
                obj.poblacioBiblioteca = query.poblacioBiblioteca;
                obj.telefonBiblioteca = query.telefonBiblioteca;
                obj.emailBiblioteca = query.emailBiblioteca;
                obj.diesPrestec = query.diesPrestec;
                obj.docsPrestec = query.docsPrestec;
                obj.pswAdministrador = query.pswAdministrador;
                obj.pswUsuari = query.pswUsuari;

                return obj;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscant configuració", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        // Modificar configuració...
        public void modificarConfiguracio(configuracioEntity configuracioAct)
        {
            var query = (from d in dc.configuracio
                         where d.id_config.Equals(1)
                         select d).Single();
            try
            {
                query.nomBiblioteca = configuracioAct.nomBiblioteca;
                query.adrecaBiblioteca = configuracioAct.adrecaBiblioteca;
                query.cpBiblioteca = configuracioAct.cpBiblioteca;
                query.poblacioBiblioteca = configuracioAct.poblacioBiblioteca;
                query.telefonBiblioteca = configuracioAct.telefonBiblioteca;
                query.emailBiblioteca = configuracioAct.emailBiblioteca;
                query.diesPrestec = configuracioAct.diesPrestec;
                query.docsPrestec = configuracioAct.docsPrestec;
                query.pswAdministrador = configuracioAct.pswAdministrador;
                query.pswUsuari = configuracioAct.pswUsuari;

                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificant configuració", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // ***********************************************************************************************
        //  Validar Login aplicacio --> valida el usuari/contrasenya al fitxer de configuració
        //    entrada: usuari i contrasenya
        //    retorn: "Error"  si la contrasenya és incorrecta
        //            usuari   si la contrasenya es correcta 
        public string validarLoginConfiguracio(string usuari, string contrasenya)
        {
            try
            {

                //Comprova si existeix la configuracio...
                iniConfiguracio();
                
                var query = (from d in dc.configuracio
                             where d.id_config.Equals(1)
                             select d).Single();
                
                string usuariRetorn = "Error";

                if (usuari == "Administrador")
                {
                    if (contrasenya.Trim() == query.pswAdministrador.Trim())
                    {
                        usuariRetorn = usuari;
                    }
                }

                if (usuari == "Usuari")
                {
                    if (contrasenya.Trim() == query.pswUsuari.Trim())
                    {
                        usuariRetorn = usuari;
                    }
                }

                return usuariRetorn;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error validant login configuració", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }

        // ***********************************************************************************************
        //  Crear un registre de configuració a la BD si no existeix
        //       (primera execució sense registre crear a la BD)
        //  Sols pot ser cridat per "validarLoginConfiguracio", que és la primera acció sobre la configuració
        //  que es realitza a l'aplicació
        private void iniConfiguracio()
        {
            var query = (from d in dc.configuracio
                         where d.id_config.Equals(1)
                         select d).SingleOrDefault();
            try
            {
                if (query == null)
                {
                    configuracio obj = new configuracio();
                    obj.id_config = 1;
                    obj.nomBiblioteca= "Nom Biblioteca";
                    obj.adrecaBiblioteca = "";
                    obj.cpBiblioteca = "";
                    obj.poblacioBiblioteca = "";
                    obj.telefonBiblioteca = "";
                    obj.emailBiblioteca = "";
                    obj.diesPrestec = 40;
                    obj.docsPrestec = 3;
                    obj.pswAdministrador = "";
                    obj.pswUsuari = "";
                    dc.configuracio.InsertOnSubmit(obj);
                    dc.SubmitChanges();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error inicialitzant configuració", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


    }//fi classe
}//fi namespace
