﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using AccesDB;
using NegociEntitats;

namespace Negoci
{
    public class documentCN
    {
        // Establim la connexió amb les classes que modelitzen la BD...
        DataClassesBibliotecaDataContext dc = new DataClassesBibliotecaDataContext();

        // ***********************************************************************************************
        //  Comprova si un document existeix
        public Boolean existeixDocument(int id_document)
        {
            try
            {
                var query = (from d in dc.documents
                             where d.id_document.Equals(id_document)
                             select d);

                if (query.Count() == 0) return false;
                return true;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error comprovant si existeix document", MessageBoxButtons.OK, MessageBoxIcon.Error); return false;
            }
        }


        
        // ***********************************************************************************************
        //  Buscar el documents per "id_document"
        //    (sols retorna un document)
        public documentsEntity buscarDocument(int id_document)
        {
            try
            {
                var query = (from d in dc.documents
                             from m in dc.materies
                             from t in dc.temes
                             where d.id_document.Equals(id_document)
                             where d.id_materia == m.id_materia
                             where d.id_tema == t.id_tema
                             select new { d.id_document, d.titol, d.signatura, d.id_format, d.id_materia,
                                          d.id_tema, d.autor, d.publicacioEditorial, d.publicacioLloc,
                                          d.publicacioAny, d.publicacioEdicio, d.coleccioNom, 
                                          d.isbn, d.dipositLegal, d.idioma, d.director, 
                                          d.interprets, d.calificacio, d.sinopsis, d.numPagines, 
                                          d.duradaMinuts, d.color, d.dataAlta, d.numUnitats, 
                                          d.numUnitatsDisponibles, d.imatge, d.actiu,
                                          m.materia, t.tema }).Single();
                
                documentsEntity obj = new documentsEntity();
                obj.id_document= query.id_document;
                obj.titol = query.titol;
                obj.signatura = query.signatura;
                obj.id_format = query.id_format;
                obj.id_materia = query.id_materia;
                obj.id_tema = query.id_tema;
                obj.tema = query.tema;
                obj.materia = query.materia;
                obj.autor = query.autor;
                obj.publicacioEditorial = query.publicacioEditorial;
                obj.publicacioLloc = query.publicacioLloc;
                obj.publicacioAny = Convert.ToInt32(query.publicacioAny);
                obj.publicacioEdicio = query.publicacioEdicio;
                obj.coleccioNom = query.coleccioNom;
                obj.isbn = query.isbn;
                obj.dipositLegal = query.dipositLegal;
                obj.idioma = query.idioma;
                obj.director = query.director;
                obj.interprets = query.interprets;
                obj.calificacio = query.calificacio;
                obj.sinopsis = query.sinopsis;
                obj.numPagines = Convert.ToInt32(query.numPagines);
                obj.duradaMinuts = Convert.ToInt32(query.duradaMinuts);
                obj.color = query.color;
                obj.dataAlta = Convert.ToDateTime(query.dataAlta);
                obj.numUnitats = Convert.ToInt32(query.numUnitats);
                obj.numUnitatsDisponibles = Convert.ToInt32(query.numUnitatsDisponibles);
                obj.imatge = query.imatge;
                obj.actiu = query.actiu;

                return obj;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error Document inexistent", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        // Afegir document...
        public void afegirDocument(documentsEntity documentAct)
        {
            try
            {
                documents obj = new documents();
                //el id_document és automàtic...
                obj.titol = documentAct.titol;
                obj.signatura = documentAct.signatura;
                obj.autor = documentAct.autor;
                obj.publicacioEditorial = documentAct.publicacioEditorial;
                obj.publicacioLloc = documentAct.publicacioLloc;
                obj.publicacioEdicio = documentAct.publicacioEdicio;
                obj.publicacioAny = documentAct.publicacioAny;
                obj.coleccioNom = documentAct.coleccioNom;
                obj.isbn = documentAct.isbn;
                obj.dipositLegal = documentAct.dipositLegal;
                obj.idioma = documentAct.idioma;
                obj.id_format = documentAct.id_format;
                obj.id_materia = documentAct.id_materia;
                obj.id_tema = documentAct.id_tema;
                obj.numPagines = documentAct.numPagines;
                obj.actiu = documentAct.actiu;

                obj.director = documentAct.director;
                obj.interprets = documentAct.interprets;
                obj.calificacio = documentAct.calificacio;
                obj.sinopsis = documentAct.sinopsis;
                obj.duradaMinuts = documentAct.duradaMinuts;
                obj.color = documentAct.color;

                obj.imatge = documentAct.imatge;
                obj.numUnitats = documentAct.numUnitats;
                obj.numUnitatsDisponibles = documentAct.numUnitatsDisponibles;
                obj.dataAlta = documentAct.dataAlta;

                dc.documents.InsertOnSubmit(obj);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


        // ***********************************************************************************************
        // Modificar document...
        public void modificarDocument(documentsEntity documentAct)
        {
            var obj = (from d in dc.documents
                       where d.id_document == documentAct.id_document
                       select d).Single();

            try
            {
                obj.titol = documentAct.titol;
                obj.signatura = documentAct.signatura;
                obj.autor = documentAct.autor;
                obj.publicacioEditorial = documentAct.publicacioEditorial;
                obj.publicacioLloc = documentAct.publicacioLloc;
                obj.publicacioEdicio = documentAct.publicacioEdicio;
                obj.publicacioAny = documentAct.publicacioAny;
                obj.coleccioNom = documentAct.coleccioNom;
                obj.isbn = documentAct.isbn;
                obj.dipositLegal = documentAct.dipositLegal;
                obj.idioma = documentAct.idioma;
                obj.id_format = documentAct.id_format;
                obj.id_materia = documentAct.id_materia;
                obj.id_tema = documentAct.id_tema;
                obj.numPagines = documentAct.numPagines;
                obj.actiu = documentAct.actiu;

                obj.director = documentAct.director;
                obj.interprets = documentAct.interprets;
                obj.calificacio = documentAct.calificacio;
                obj.sinopsis = documentAct.sinopsis;
                obj.duradaMinuts = documentAct.duradaMinuts;
                obj.color = documentAct.color;

                obj.imatge = documentAct.imatge;
                obj.numUnitats = documentAct.numUnitats;
                obj.numUnitatsDisponibles = documentAct.numUnitatsDisponibles;
                obj.dataAlta = documentAct.dataAlta;

                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    

        // ***********************************************************************************************
        // Eliminar document...
        public void eliminarDocument(int id_document)
        {
            try
            {
                var query = (from d in dc.documents
                             where d.id_document == id_document
                             select d).Single();
                dc.documents.DeleteOnSubmit(query);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        //  Llistar els documents per "id_document"
        //    (sols retorna un document)
        //    SIN USO
        public List<documentsEntity> llistarDocumentsId(int id_document)
        {
            try
            {
                List<documentsEntity> llista = new List<documentsEntity>();
                var query = (from d in dc.documents
                             where d.id_document.Equals(id_document)
                             select d);

                foreach (var d in query)
                {
                    documentsEntity obj = new documentsEntity();
                    obj.id_document = d.id_document;
                    obj.signatura = d.signatura;
                    obj.titol = d.titol;
                    obj.autor = d.autor;
                    obj.id_format = d.id_format;
                    obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                    obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                    obj.actiu = d.actiu;
                    llista.Add(obj);
                }
                return llista;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        //  Llistar els documents 
        //  tipoconsulta = 1 = Tots
        //                 2 = Buscar per Id
        //                 3 = buscar per titol
        //                 4 = buscar per autor
        //                 5 = buscar per tema
        public List<documentsEntity> llistarDocuments(int tipoConsulta, string valorBuscat)
        {
            try
            {
                List<documentsEntity> llista = new List<documentsEntity>();

                // 1=Tots...
                if (tipoConsulta == 1)
                {
                    var query = (from d in dc.documents
                                 where true
                                 select d);

                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                // 2=Buscar per id_document...
                if (tipoConsulta == 2)
                {
                    int valorn = 0;
                    valorn = Convert.ToInt32(valorBuscat);
                    var query = (from d in dc.documents
                                 where d.id_document.Equals(valorn)
                                 select d);

                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                
                // 3=Buscar per titol...
                if (tipoConsulta == 3)
                {
                    var query = from d in dc.documents
                                where (d.titol.Contains(valorBuscat) )
                                 orderby d.id_document
                                 select d;
                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                // 4=Buscar per autor...
                if (tipoConsulta == 4)
                {
                    var query = from d in dc.documents
                                where (d.autor.Contains(valorBuscat) || d.director.Contains(valorBuscat))
                                orderby d.id_document
                                select d;
                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                // 5=Buscar per ISBN...
                if (tipoConsulta == 5)
                {
                    var query = from d in dc.documents
                                where (d.isbn.Contains(valorBuscat))
                                orderby d.id_document
                                select d;
                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                // 6=Buscar per signatura...
                if (tipoConsulta == 6)
                {
                    var query = from d in dc.documents
                                where (d.signatura.Contains(valorBuscat))
                                orderby d.id_document
                                select d;
                    foreach (var d in query)
                    {
                        documentsEntity obj = new documentsEntity();
                        obj.id_document = d.id_document;
                        obj.signatura = d.signatura;
                        obj.titol = d.titol;
                        obj.autor = d.autor;
                        obj.id_format = d.id_format;
                        obj.publicacioAny = Convert.ToInt32(d.publicacioAny);
                        obj.numUnitatsDisponibles = Convert.ToInt32(d.numUnitatsDisponibles);
                        obj.actiu = d.actiu;
                        llista.Add(obj);
                    }
                }

                
                //Retorn...
                return llista;
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }

        }



        // ***********************************************************************************************
        // Cambia unitats disponibles d'un document...
        //  sols controla una unitat per document
        //   unitats = 0 = Es fa un prestec (fica a 0 les unitats disponibles)
        //             1 = Es fa una devolució (fica a 1 les unitats disponibles)
        public void canviarUnitatsDocument(int id_document, int unitats)
        {
            var obj = (from d in dc.documents
                       where d.id_document == id_document
                       select d).Single();

            try
            {
                obj.numUnitatsDisponibles = unitats;
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error canviar unitats document", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


            
    } //fi classe
} //fi namespace
