﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using AccesDB;
using NegociEntitats;

using System.Linq.Expressions;

namespace Negoci
{
    public class prestecsCN
    {
        // Establim la connexió amb les classes que modelitzen la BD...
        DataClassesBibliotecaDataContext dc = new DataClassesBibliotecaDataContext();

        // ***********************************************************************************************
        //  Buscar el prestecs per "id_prestec"
        //    (sols retorna un prestec)
        public prestecsEntity buscarPrestec(int id_prestec)
        {
            try
            {
                var query = (from p in dc.prestecs
                             from d in dc.documents
                             from s in dc.socis
                             where p.id_prestec.Equals(id_prestec)
                             where p.id_document == d.id_document
                             where p.id_soci== s.id_soci 
                             select new { p.id_prestec, p.id_document, p.id_soci, p.dataIniciPrestec, 
                                          p.dataFiPrestec, p.dataRetornPrestec, p.numRenovacions,
                                          p.retornat, d.titol, d.autor, d.signatura, d.isbn, d.id_format,
                                          s.nom, s.cognoms }).Single();

                prestecsEntity obj = new prestecsEntity();
                obj.id_prestec = query.id_prestec;
                obj.id_document = query.id_document;
                obj.id_soci = query.id_soci;
                obj.dataIniciPrestec = Convert.ToDateTime(query.dataIniciPrestec);
                obj.dataFiPrestec = Convert.ToDateTime(query.dataFiPrestec);
                obj.dataRetornPrestec = Convert.ToDateTime(query.dataRetornPrestec);
                obj.numRenovacions = Convert.ToInt32(query.numRenovacions);
                obj.retornat = query.retornat;
                obj.autor = query.autor;
                obj.titol = query.titol;
                obj.signatura = query.signatura;
                obj.isbn = query.isbn;
                obj.id_format = query.id_format;
                obj.nom = query.nom;
                obj.cognoms = query.cognoms;

                return obj;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscarPréstec", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }

        // ***********************************************************************************************
        //  Buscar el prestec per "id_document" i que estigui "no retornat"
        //    (sols retorna un prestec)
        public prestecsEntity buscarPrestecIdDocument(int id_document)
        {
            try
            {
                var query = (from p in dc.prestecs
                             where p.id_document == id_document
                             where p.retornat == false
                             select p).FirstOrDefault();


                prestecsEntity obj = new prestecsEntity();
                if (query == null)  return null;

                obj.id_prestec = query.id_prestec;
                obj.id_document = query.id_document;
                obj.id_soci = query.id_soci;
                obj.dataIniciPrestec = Convert.ToDateTime(query.dataIniciPrestec);
                obj.dataFiPrestec = Convert.ToDateTime(query.dataFiPrestec);
                obj.dataRetornPrestec = Convert.ToDateTime(query.dataRetornPrestec);
                obj.numRenovacions = Convert.ToInt32(query.numRenovacions);
                obj.retornat = query.retornat;
                return obj;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscar préstec IdDocument", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        // Afegir préstec...
        public void afegirPrestec(prestecsEntity prestecAct)
        {
            try
            {
                prestecs obj = new prestecs();
                //el id_prestec és automàtic...
                obj.id_document = prestecAct.id_document;
                obj.id_soci = prestecAct.id_soci;
                obj.dataIniciPrestec = prestecAct.dataIniciPrestec;
                obj.dataFiPrestec = prestecAct.dataFiPrestec;
                //obj.dataRetornPrestec = prestecAct.dataRetornPrestec;
                obj.numRenovacions = prestecAct.numRenovacions;
                obj.retornat = prestecAct.retornat;

                dc.prestecs.InsertOnSubmit(obj);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error afegir préstec", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


        // ***********************************************************************************************
        // Modificar Préstec...
        public void modificarPrestec(prestecsEntity prestecAct)
        {
            var obj = (from d in dc.prestecs
                       where d.id_prestec == prestecAct.id_prestec
                       select d).Single();

            try
            {
                obj.id_document = prestecAct.id_document;
                obj.id_soci = prestecAct.id_soci;
                obj.dataIniciPrestec = prestecAct.dataIniciPrestec;
                obj.dataFiPrestec = prestecAct.dataFiPrestec;
                if (isDateNull(prestecAct.dataRetornPrestec) == false)
                {
                    obj.dataRetornPrestec = prestecAct.dataRetornPrestec;
                }
                obj.numRenovacions = prestecAct.numRenovacions;
                obj.retornat = prestecAct.retornat;

                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificar préstec", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        // Eliminar préstec...
        public void eliminarPrestec(int id_prestec)
        {
            try
            {
                var query = (from d in dc.prestecs
                             where d.id_prestec == id_prestec
                             select d).Single();
                dc.prestecs.DeleteOnSubmit(query);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error eliminar préstec", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // ***********************************************************************************************
        //  Busca quants prestecs actius te un soci
        public int buscarPrestecsActiusIdSoci(int id_soci)
        {
            try
            {
                var query = (from p in dc.prestecs
                             where p.id_soci == id_soci
                             where p.retornat == false
                             select p);

                if (query == null) return 0;
                return query.Count();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscar prestecs id-soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return 0;
            }
        }



        // ***********************************************************************************************
        //  Llistar els prestecs d'un Soci (per id_soci)
        public List<prestecsEntity> llistarPrestecsIdSoci(int id_soci)
        {
            try
            {
                List<prestecsEntity> llista = new List<prestecsEntity>();
                var query = (from p in dc.prestecs
                             from d in dc.documents
                             where p.id_soci == id_soci
                             where p.id_document == d.id_document
                             select new
                             {
                                 p.id_prestec,
                                 p.id_document,
                                 p.id_soci,
                                 p.dataIniciPrestec,
                                 p.dataFiPrestec,
                                 p.dataRetornPrestec,
                                 p.numRenovacions,
                                 p.retornat,
                                 d.titol,
                                 d.autor,
                                 d.signatura,
                                 d.isbn,
                                 d.id_format
                             });

                foreach (var d in query)
                {
                    prestecsEntity obj = new prestecsEntity();
                    obj.id_prestec = d.id_prestec;
                    obj.id_document = d.id_document;
                    obj.id_soci = d.id_soci;
                    obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                    obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                    obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                    obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                    obj.retornat = d.retornat;
                    obj.autor = d.autor;
                    obj.titol = d.titol;
                    obj.signatura = d.signatura;
                    obj.isbn = d.isbn;
                    obj.id_format = d.id_format;
                    llista.Add(obj);
                }
                return llista;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error llistar prestecs id-soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        //  Llistar els prestecs 
        //  tipoconsulta = 1 = Tots
        //                 2 = Buscar per Id prestec
        //                 3 = buscar per id document
        //                 4 = buscar per id soci
        //                 5 = buscar per titol document
        //                 6 = buscar per nom soci
        //                 7 = pendents de retornar
        public List<prestecsEntity> llistarPrestecs(int tipoConsulta, string valorBuscat)
        {
            try
            {
                List<prestecsEntity> llista = new List<prestecsEntity>();

                // 1=Tots...
                if (tipoConsulta == 1)
                {
                    //exemple amb join
                    //var query = ( from p in dc.prestecs 
                    //              join d in dc.documents on p.id_document equals d.id_document
                    //              join s in dc.socis on p.id_soci equals s.id_soci 
                                        
                    var query = (from p in dc.prestecs
                                 from d in dc.documents
                                 from s in dc.socis
                                 where p.id_document == d.id_document 
                                 where p.id_soci == s.id_soci
                                 select new
                                 {
                                     p.id_prestec,
                                     p.id_document,
                                     p.id_soci,
                                     p.dataIniciPrestec,
                                     p.dataFiPrestec,
                                     p.dataRetornPrestec,
                                     p.numRenovacions,
                                     p.retornat,
                                     d.titol,
                                     d.autor,
                                     d.signatura,
                                     d.isbn,
                                     d.id_format,
                                     s.nom,
                                     s.cognoms
                                 });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                // 2=Per Id prestec...
                if (tipoConsulta == 2)
                {
                    int id_prestec = Convert.ToInt32(valorBuscat);

                    var query = (from p in dc.prestecs
                                  from d in dc.documents
                                  from s in dc.socis
                                  where p.id_document == d.id_document
                                  where p.id_soci == s.id_soci
                                  where p.id_prestec == id_prestec
                                  select new
                                  {
                                      p.id_prestec,
                                      p.id_document,
                                      p.id_soci,
                                      p.dataIniciPrestec,
                                      p.dataFiPrestec,
                                      p.dataRetornPrestec,
                                      p.numRenovacions,
                                      p.retornat,
                                      d.titol,
                                      d.autor,
                                      d.signatura,
                                      d.isbn,
                                      d.id_format,
                                      s.nom,
                                      s.cognoms 
                                  });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                // 3=Per Id document...
                if (tipoConsulta == 3)
                {
                    int id_document = Convert.ToInt32(valorBuscat);

                    var query  = (from p in dc.prestecs
                                  from d in dc.documents
                                  from s in dc.socis
                                  where p.id_document == d.id_document
                                  where p.id_soci == s.id_soci
                                  where p.id_document == id_document
                                  select new
                                  {
                                      p.id_prestec,
                                      p.id_document,
                                      p.id_soci,
                                      p.dataIniciPrestec,
                                      p.dataFiPrestec,
                                      p.dataRetornPrestec,
                                      p.numRenovacions,
                                      p.retornat,
                                      d.titol,
                                      d.autor,
                                      d.signatura,
                                      d.isbn,
                                      d.id_format,
                                      s.nom,
                                      s.cognoms 
                                  });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                // 4=Per id soci...
                if (tipoConsulta == 4)
                {
                    int id_soci = Convert.ToInt32(valorBuscat);

                    var query = (from p in dc.prestecs
                                 from d in dc.documents
                                 from s in dc.socis
                                 where p.id_document == d.id_document
                                 where p.id_soci == s.id_soci
                                 where p.id_soci == id_soci
                                 select new
                                 {
                                     p.id_prestec,
                                     p.id_document,
                                     p.id_soci,
                                     p.dataIniciPrestec,
                                     p.dataFiPrestec,
                                     p.dataRetornPrestec,
                                     p.numRenovacions,
                                     p.retornat,
                                     d.titol,
                                     d.autor,
                                     d.signatura,
                                     d.isbn,
                                     d.id_format,
                                     s.nom,
                                     s.cognoms
                                 });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                // 5=Per titol document...
                if (tipoConsulta == 5)
                {
                    var query = (from p in dc.prestecs
                                  from d in dc.documents
                                  from s in dc.socis
                                  where p.id_document == d.id_document
                                  where p.id_soci == s.id_soci
                                  where d.titol.Contains(valorBuscat)
                                  select new
                                  {
                                      p.id_prestec,
                                      p.id_document,
                                      p.id_soci,
                                      p.dataIniciPrestec,
                                      p.dataFiPrestec,
                                      p.dataRetornPrestec,
                                      p.numRenovacions,
                                      p.retornat,
                                      d.titol,
                                      d.autor,
                                      d.signatura,
                                      d.isbn,
                                      d.id_format,
                                      s.nom,
                                      s.cognoms 
                                  });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }


                // 6=Per nom soci...
                if (tipoConsulta == 6)
                {
                    int id_soci = Convert.ToInt32(valorBuscat);

                    var query = (from p in dc.prestecs
                                  from d in dc.documents
                                  from s in dc.socis
                                  where p.id_document == d.id_document
                                  where p.id_soci == s.id_soci
                                  where s.cognoms.Contains(valorBuscat)
                                  select new
                                  {
                                      p.id_prestec,
                                      p.id_document,
                                      p.id_soci,
                                      p.dataIniciPrestec,
                                      p.dataFiPrestec,
                                      p.dataRetornPrestec,
                                      p.numRenovacions,
                                      p.retornat,
                                      d.titol,
                                      d.autor,
                                      d.signatura,
                                      d.isbn,
                                      d.id_format,
                                      s.nom,
                                      s.cognoms 
                                  });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                // 7=Pendents de retornar...
                if (tipoConsulta == 7)
                {
                    var query = (from p in dc.prestecs
                                  from d in dc.documents
                                  from s in dc.socis
                                  where p.id_document == d.id_document
                                  where p.id_soci == s.id_soci
                                  where p.retornat == false
                                  select new
                                  {
                                      p.id_prestec,
                                      p.id_document,
                                      p.id_soci,
                                      p.dataIniciPrestec,
                                      p.dataFiPrestec,
                                      p.dataRetornPrestec,
                                      p.numRenovacions,
                                      p.retornat,
                                      d.titol,
                                      d.autor,
                                      d.signatura,
                                      d.isbn,
                                      d.id_format,
                                      s.nom,
                                      s.cognoms
                                  });

                    foreach (var d in query)
                    {
                        prestecsEntity obj = new prestecsEntity();
                        obj.id_prestec = d.id_prestec;
                        obj.id_document = d.id_document;
                        obj.id_soci = d.id_soci;
                        obj.dataIniciPrestec = Convert.ToDateTime(d.dataIniciPrestec);
                        obj.dataFiPrestec = Convert.ToDateTime(d.dataFiPrestec);
                        obj.dataRetornPrestec = Convert.ToDateTime(d.dataRetornPrestec);
                        obj.numRenovacions = Convert.ToInt32(d.numRenovacions);
                        obj.retornat = d.retornat;
                        obj.autor = d.autor;
                        obj.titol = d.titol;
                        obj.signatura = d.signatura;
                        obj.isbn = d.isbn;
                        obj.id_format = d.id_format;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        llista.Add(obj);
                    }
                }

                //Retorn...
                return llista;
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error llistarPrestecs", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }

        }

        // ***********************************************************************************************
        //  Renovació
        //     Calcular la nova data de fi de préstec
        //        entrada: datafiprestec actual, diesprestec (segon configuració)
        //        retorn:  nova data fi prestec
        public string calcularRenovacioPrestec(string dataFiPrestec, int diesPrestec)
        {
            DateTime dataFiPrestecNw = new DateTime();

            dataFiPrestecNw = DateTime.Parse(dataFiPrestec); 
            if (diesPrestec == 30)
            {
                dataFiPrestecNw = dataFiPrestecNw.AddMonths(1);
            }
            else
            {
                dataFiPrestecNw = dataFiPrestecNw.AddDays(diesPrestec);
            }

            return dataFiPrestecNw.ToString("dd/MM/yyyy");
        }

        // ***********************************************************************************************
        // Renovació
        //     guarda la nova data de retorn  (renovació per Web)
        public void renovacioPrestecWeb(prestecsEntity prestecAct)
        {
            var obj = (from d in dc.prestecs
                       where d.id_prestec == prestecAct.id_prestec
                       select d).Single();

            try
            {
                obj.dataFiPrestec = prestecAct.dataFiPrestec;
                obj.numRenovacions = prestecAct.numRenovacions;
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error renovacio préstec web", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }




        //*********************************************************************************************************+
        //Comprovar si la data es null
        private Boolean isDateNull(DateTime data)
        {
            if (data.ToString("dd/MM/yyyy") == "01/01/0001")
            {
                return true;
            }
            else
            {
                return false;
            }
        }



    } //fi classe
} //fi namespace
