﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.Globalization;
using AccesDB;
using NegociEntitats;

namespace Negoci
{
    public class socisCN
    {
        // Establim la connexió amb les classes que modelitzen la BD...
        DataClassesBibliotecaDataContext dc = new DataClassesBibliotecaDataContext();

        // ***********************************************************************************************
        //  Comprova si existeix un soci
        public Boolean existeixSoci(int id_soci)
        {
            try
            {
                var query = (from d in dc.socis
                             where d.id_soci.Equals(id_soci)
                             select d);

                if (query.Count() == 0) return false;
                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error comprovant existencia Soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return false;
            }
        }

        
        // ***********************************************************************************************
        //  Buscar el socis per "id_soci"
        //    (sols retorna un soci)
        public socisEntity buscarSoci(int id_soci)
        {
            try
            {
                var query = (from d in dc.socis
                             where d.id_soci.Equals(id_soci)
                             select d).Single();
                socisEntity obj = new socisEntity();
                obj.id_soci = query.id_soci;
                obj.nom = query.nom;
                obj.cognoms = query.cognoms;
                obj.sexe = query.sexe;
                obj.dni = query.dni;
                obj.dataNaixement = Convert.ToDateTime(query.dataNaixement);
                obj.id_tipusSoci = Convert.ToInt32(query.id_tipusSoci);  
                obj.adreca = query.adreca;
                obj.cp = query.cp;
                obj.poblacio = query.poblacio;
                obj.provincia = query.provincia;
                obj.pais = query.pais;
                obj.telefon = query.telefon;
                obj.email = query.email;
                obj.dataAlta = Convert.ToDateTime(query.dataAlta);
                obj.dataBaixa = Convert.ToDateTime(query.dataBaixa);
                obj.esBaixa = query.esBaixa;
                obj.contrasenya = query.contrasenya;
                obj.actiu = query.actiu;
                return obj;

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscant soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }


        // ***********************************************************************************************
        // Afegir soci...
        public void afegirSoci(socisEntity sociAct)
        {
            try
            {
                socis obj = new socis();
                //el id_soci és automàtic...
                obj.nom = sociAct.nom;
                obj.cognoms = sociAct.cognoms;
                obj.sexe = sociAct.sexe;
                obj.dni = sociAct.dni;
                obj.dataNaixement = sociAct.dataNaixement;
                obj.id_tipusSoci = sociAct.id_tipusSoci;
                obj.adreca = sociAct.adreca;
                obj.cp = sociAct.cp;
                obj.poblacio = sociAct.poblacio;
                obj.provincia = sociAct.provincia;
                obj.pais = sociAct.pais;
                obj.telefon = sociAct.telefon;
                obj.email = sociAct.email;
                obj.dataAlta = sociAct.dataAlta;
                if (isDateNull(sociAct.dataBaixa) == false)
                {
                    obj.dataBaixa = sociAct.dataBaixa;
                }
                obj.esBaixa = sociAct.esBaixa;
                obj.contrasenya = sociAct.contrasenya;
                obj.actiu = sociAct.actiu;

                dc.socis.InsertOnSubmit(obj);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error afegint soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        // Modificar soci...
        public void modificarSoci(socisEntity sociAct)
        {
            var obj = (from d in dc.socis
                       where d.id_soci == sociAct.id_soci
                       select d).Single();

            try
            {
                obj.nom = sociAct.nom;
                obj.cognoms = sociAct.cognoms;
                obj.sexe = sociAct.sexe;
                obj.dni = sociAct.dni;
                obj.dataNaixement = sociAct.dataNaixement;
                obj.id_tipusSoci = sociAct.id_tipusSoci;
                obj.adreca = sociAct.adreca;
                obj.cp = sociAct.cp;
                obj.poblacio = sociAct.poblacio;
                obj.provincia = sociAct.provincia;
                obj.pais = sociAct.pais;
                obj.telefon = sociAct.telefon;
                obj.email = sociAct.email;
                obj.dataAlta = sociAct.dataAlta;
                if (isDateNull(sociAct.dataBaixa) == false )
                {
                    obj.dataBaixa = sociAct.dataBaixa;
                }
                obj.esBaixa = sociAct.esBaixa;
                obj.contrasenya = sociAct.contrasenya;
                obj.actiu = sociAct.actiu;

                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificant soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        // ***********************************************************************************************
        // Modificar soci desde Web...
        public void modificarSociWeb(int id_soci, string telefon, string email)
        {
            var obj = (from d in dc.socis
                       where d.id_soci == id_soci
                       select d).Single();

            try
            {
                obj.telefon = telefon;
                obj.email = email;
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificant soci Web", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        // Eliminar soci...
        public void eliminarSoci(int id_soci)
        {
            try
            {
                var query = (from d in dc.socis
                             where d.id_soci == id_soci
                             select d).Single();
                dc.socis.DeleteOnSubmit(query);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error eliminant soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }



        // ***********************************************************************************************
        //  Llistar els socis 
        //  tipoconsulta = 1 = Tots
        //                 2 = Buscar per Id
        //                 3 = buscar per cognoms
        //                 4 = buscar per dni
        public List<socisEntity> llistarSocis(int tipoConsulta, string valorBuscat)
        {
            try
            {
                List<socisEntity> llista = new List<socisEntity>();

                // 1=Tots...
                if (tipoConsulta == 1)
                {
                    var query = (from d in dc.socis
                                 where true
                                 select d);

                    foreach (var d in query)
                    {
                        socisEntity obj = new socisEntity();
                        obj.id_soci = d.id_soci;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        obj.dni = d.dni;
                        llista.Add(obj);
                    }
                }

                // 2=Buscar per id_soci...
                if (tipoConsulta == 2)
                {
                    int valorn = 0;
                    valorn = Convert.ToInt32(valorBuscat);
                    var query = (from d in dc.socis
                                 where d.id_soci.Equals(valorn)
                                 select d);

                    foreach (var d in query)
                    {
                        socisEntity obj = new socisEntity();
                        obj.id_soci = d.id_soci;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        obj.dni = d.dni;
                        llista.Add(obj);
                    }
                }


                // 3=Buscar per cognoms...
                if (tipoConsulta == 3)
                {
                    var query = from d in dc.socis
                                where (d.cognoms.Contains(valorBuscat))
                                orderby d.id_soci
                                select d;
                    foreach (var d in query)
                    {
                        socisEntity obj = new socisEntity();
                        obj.id_soci = d.id_soci;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        obj.dni = d.dni;
                        llista.Add(obj);
                    }
                }

                // 4=Buscar per dni...
                if (tipoConsulta == 4)
                {
                    var query = from d in dc.socis
                                where d.dni.Equals(valorBuscat)
                                orderby d.id_soci
                                select d;
                    foreach (var d in query)
                    {
                        socisEntity obj = new socisEntity();
                        obj.id_soci = d.id_soci;
                        obj.nom = d.nom;
                        obj.cognoms = d.cognoms;
                        obj.dni = d.dni;
                        llista.Add(obj);
                    }
                }

                //Retorn...
                return llista;
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }

        }


        // ***********************************************************************************************
        //  Valida el login d'un soci per num.soci (id_soci) ó email i la seva contrasenya
        //     Si existeix --> retorna el id_soci
        //     si no existeix --> retorna 0   

        public int validarLoginSoci(string usuari, string contrasenya)
        {
            try
            {

                //Validar login usuari per num_soci  (id_soci)
                if (isNumeric(usuari))
                {
                    int id_soci = Convert.ToInt32(usuari);
                    var query = (from d in dc.socis
                                 where d.id_soci == id_soci && d.contrasenya == contrasenya
                                 select d);

                    if (query.Count() != 0) //Si existeix 
                    {
                        return query.First().id_soci;
                    }
                    else // no existeix
                    {
                        return 0;
                    }
                }
                else
                {
                    var query = (from d in dc.socis
                                 where d.email == usuari && d.contrasenya == contrasenya
                                 select d);

                    if (query.Count() != 0) //Si existeix 
                    {
                        return query.First().id_soci; 
                    }
                    else // no existeix
                    {
                        return 0;
                    }
                }
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error validant login soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return 0;
            }
        }


        // ***********************************************************************************************
        //  Comprova si ja existeix un email
        public Boolean existeixEmailSoci(int id_soci, string email)
        {
            try
            {
                email = email.Trim();
                //email buit..
                if (String.IsNullOrEmpty(email) == true) return false;

                var query = (from d in dc.socis
                             where d.email.Equals(email)
                             select d);

                //No hi ha cap email igual
                if (query.Count() == 0) return false;

                //Hi ha un email igual, però és el del soci
                if (id_soci == query.First().id_soci) return false;

                //Hi ha un email igual...
                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error comprovant existència email Soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return false;
            }
        }


        // ***********************************************************************************************
        //  Comprova si ja existeix un dni
        public Boolean existeixDniSoci(int id_soci, string dni)
        {
            try
            {
                dni = dni.Trim();
                //dni buit..
                if (String.IsNullOrEmpty(dni) == true) return false;

                var query = (from d in dc.socis
                             where d.dni.Equals(dni)
                             select d);

                //No hi ha cap dni igual
                if (query.Count() == 0) return false;

                //Hi ha un dni igual, però és el del soci
                if (id_soci == query.First().id_soci) return false;

                //Hi ha un dni igual...
                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error comprovant existència dni Soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return false;
            }
        }


        // ***********************************************************************************************
        // Canviar contrasenya soci desde Web...
        public void canviarContrasenyaSociWeb(int id_soci, string contrasenya)
        {
            var obj = (from d in dc.socis
                       where d.id_soci == id_soci
                       select d).Single();

            try
            {
                obj.contrasenya = contrasenya;
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificant contrasenya soci Web", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //*********************************************************************************************************+
        //Validar numero
        private Boolean isNumeric(string numero)
        {
            int number = 0;
            if (int.TryParse(numero.Trim(), out number)) return true;
            return false;
        }


        //*********************************************************************************************************+
        //Comprovar si la data es null
        private Boolean isDateNull(DateTime data)
        {
           if (data.ToString("dd/MM/yyyy") == "01/01/0001")
            {
                return true;
            }
            else
            {
                return false;
            }
        }



    } //fi classe
} //fi namespace
