﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using AccesDB;
using NegociEntitats;

namespace Negoci
{
    public class tipusSocisCN
    {
        // Establim la connexió amb les classes que modelitzen la BD...
        DataClassesBibliotecaDataContext dc = new DataClassesBibliotecaDataContext();

        // ***********************************************************************************************
        //  Comprova si existeix un id_tipus_soci
        public Boolean existeixTipusSoci(int id_tipusSoci)
        {
            try
            {
                var query = (from d in dc.tipus_socis
                             where d.id_tipusSoci.Equals(id_tipusSoci)
                             select d);

                if (query.Count() == 0) return false;
                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error comprovant existencia tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return false;
            }
        }

        
        // ***********************************************************************************************
        //  Buscar el registre
        public tipusSocisEntity buscarTipusSoci(int id_tipusSoci)
        {
            try
            {
                    var query = (from d in dc.tipus_socis
                                 where d.id_tipusSoci.Equals(id_tipusSoci)
                                 select d).Single();
                    tipusSocisEntity obj = new tipusSocisEntity();
                    obj.id_tipusSoci = query.id_tipusSoci;
                    obj.tipusSoci = query.tipusSoci;
                    return obj;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error buscar tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }

        // ***********************************************************************************************
        // Afegir registre...
        public void afegirTipusSoci(tipusSocisEntity tipusSociAct)
        {
            try
            {
                tipus_socis obj = new tipus_socis();
                obj.id_tipusSoci = tipusSociAct.id_tipusSoci;
                obj.tipusSoci = tipusSociAct.tipusSoci;

                dc.tipus_socis.InsertOnSubmit(obj);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error afegint tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        // Modificar registre...
        public void modificarTipusSoci(tipusSocisEntity tipusSociAct)
        {
            var obj = (from d in dc.tipus_socis
                       where d.id_tipusSoci == tipusSociAct.id_tipusSoci
                       select d).Single();

            try
            {
                obj.id_tipusSoci = tipusSociAct.id_tipusSoci;
                obj.tipusSoci = tipusSociAct.tipusSoci;
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error modificant tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        // Eliminar registre...
        public void eliminarTipusSoci(int id_tipusSoci)
        {
            try
            {
                var query = (from d in dc.tipus_socis
                             where d.id_tipusSoci == id_tipusSoci
                             select d).Single();
                dc.tipus_socis.DeleteOnSubmit(query);
                dc.SubmitChanges();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error eliminant tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        // ***********************************************************************************************
        //  Llistar els tipus Socis
        public List<tipusSocisEntity> llistarTipusSocis()
        {
            try
            {
                List<tipusSocisEntity> llista = new List<tipusSocisEntity>();
                var query = (from d in dc.tipus_socis
                             where true
                             select d);
                foreach (var d in query)
                {
                    tipusSocisEntity obj = new tipusSocisEntity();
                    obj.id_tipusSoci = d.id_tipusSoci;
                    obj.tipusSoci = d.tipusSoci;
                    llista.Add(obj);
                }
                return llista;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error Llistar Tipus Socis", MessageBoxButtons.OK, MessageBoxIcon.Error); return null;
            }
        }




    }//fi classe
}//fi namespace
