﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.IO;
using Negoci;
using Terminal.ServiceReference1;
using NegociEntitats;

namespace Terminal
{
    public partial class frm2Document : Form
    {
        private ServiceReference1.Service1Client clientWCF = new ServiceReference1.Service1Client(); // Instancia del servei WCF
        //private simulawcfCN clientWCF = new simulawcfCN(); // simula WCF

        private List<formatsEntity> ListFormats;
        private List<materiesEntity> ListMateries;
        private List<temesEntity> ListTemes;
        private List<idiomesEntity> ListIdiomes;
        private documentsEntity documentAct = new documentsEntity();
        private int id_documentAct = 0;
        private string pathPrograma="";
         
        public frm2Document(int id_document)
        {
            InitializeComponent();
            id_documentAct = id_document;
        }

        private void frm2Document_Load(object sender, EventArgs e)
        {
            //Carregar combos...
            carregarCmbFormats();
            carregarCmbMateries();
            carregarCmbTemes();
            carregarCmbIdiomes();
            buscarPathPrograma();

            if (id_documentAct == 0)
            {
                altaPerDefecte();
            }
            else
            {
                buscarDocument();
            }

            moureCamps_a_Pantalla();


            //Desactivar opcions segons usuari actiu
            //  si rol=2 (usuari normal), desactivar opcions
            gblParametres parametres = gblParametres.Instance();
            if (parametres.usuariRol == 2)
            {
                acceptarButton.Enabled = false;
                eliminarButton.Enabled = false;
            }

        }

        private void acceptarButton_Click(object sender, EventArgs e)
        {
            //Valida que els camps estiguin Ok...
            Boolean correcte = true;
            correcte = validarCamps();
            if (!correcte)
            {
                return;
            }

            if (id_documentAct == 0)
            {
                moureCamps();
                afegirDocument();
                this.Dispose();
            }
            else
            {
                moureCamps();
                modificarDocument();
                this.Dispose();
            }
        }

        //********************************************************************************************
        private void eliminarButton_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("Confirmar l'eliminació", "Eliminar Document", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

            if (result == DialogResult.OK)
            {
                eliminarDocument();
                this.Dispose();
            }
        }

        private void cancelarButton_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }


        //*********************************************************************************************************+
        //Validar els camps introduits...
        private Boolean validarCamps()
        {
            Boolean correcte = true;

            //Validar titol
            if (String.IsNullOrEmpty(txtTitol.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar TITOL", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtTitol.Focus();
                return false;
            }

            //Validar Autor
            if (String.IsNullOrEmpty(txtAutor.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar AUTOR", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtAutor.Focus();
                return false;
            }

            //Validar format
            if (String.IsNullOrEmpty(cmbId_format.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar FORMAT", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                cmbId_format.Focus();
                return false;
            }

            //Validar materia
            if (String.IsNullOrEmpty(cmbId_materia.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar MATERIA", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                cmbId_materia.Focus();
                return false;
            }

            //Validar tema
            if (String.IsNullOrEmpty(cmbId_tema.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar TEMA", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                cmbId_tema.Focus();
                return false;
            }

            //Validar idioma
            if (String.IsNullOrEmpty(cmbIdioma.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar IDIOMA", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                cmbIdioma.Focus();
                return false;
            }


            //Validar anyPublicacio
            if (String.IsNullOrEmpty(txtPublicacioAny.Text) == false) 
            {
                if (this.isNumeric(txtPublicacioAny.Text) == false)
                {
                    DialogResult result = MessageBox.Show("ANY PUBLICACIO sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtPublicacioAny.Focus();
                    return false; 
                }
            }

            //Validar numPagines
            if (String.IsNullOrEmpty(txtNumPagines.Text) == false)
            {
                if (this.isNumeric(txtNumPagines.Text) == false)
                {
                    DialogResult result = MessageBox.Show("NUM.PÀGINES sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtNumPagines.Focus();
                    return false;
                }
            }

            //Validar duradaMinuts
            if (String.IsNullOrEmpty(txtDuradaMinuts.Text) == false)
            {
                if (this.isNumeric(txtDuradaMinuts.Text) == false)
                {
                    DialogResult result = MessageBox.Show("DURADA (minuts) sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDuradaMinuts.Focus();
                    return false;
                }
            }

            //Validar color
            if (String.IsNullOrEmpty(cmbColor.Text) == false)
            {
                if (cmbColor.Text != "S" && cmbColor.Text != "N")
                {
                    DialogResult result = MessageBox.Show("COLOR sols admet S ó N", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    cmbColor.Focus();
                    return false;
                }
            }

            //Validar numUnitats
            if (String.IsNullOrEmpty(txtNumUnitats.Text) == false)
            {
                if (this.isNumeric(txtNumUnitats.Text) == false)
                {
                    DialogResult result = MessageBox.Show("NUM.UNITATS sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtNumUnitats.Focus();
                    return false;
                }
            }

            //Validar numUnitatsDisponibles
            if (String.IsNullOrEmpty(txtNumUnitatsDisponibles.Text) == false)
            {
                if (this.isNumeric(txtNumUnitatsDisponibles.Text) == false)
                {
                    DialogResult result = MessageBox.Show("NUM.UNITATS DISPONIBLES sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtNumUnitatsDisponibles.Focus();
                    return false;
                }
                if (Convert.ToInt32(txtNumUnitatsDisponibles.Text) > 1)
                {
                    DialogResult result = MessageBox.Show("NUM.UNITATS DISPONIBLES sols admet 1 una unitat disponible", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtNumUnitatsDisponibles.Focus();
                    return false;
                }
            }

            //Validar dataAlta
            if (String.IsNullOrEmpty(txtDataAlta.Text) == false)
            {
                if (this.isDate(txtDataAlta.Text) == false)
                {
                    DialogResult result = MessageBox.Show("DATA ALTA Incorrecte", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDataAlta.Focus();
                    return false;
                }
            }
            if (String.IsNullOrEmpty(txtDataAlta.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar DATA ALTA", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtDataAlta.Focus();
                return false;
            }


            //Retorn
            return correcte;
        }

        //*********************************************************************************************************+
        //Mou els camps a la pantalla ...
        private void moureCamps_a_Pantalla()
        {
            txtId_document.Text = Convert.ToString(documentAct.id_document);
            if (documentAct.id_document == 0) txtId_document.Text = "*** nou ***";
            txtTitol.Text = documentAct.titol;
            txtSignatura.Text = documentAct.signatura;
            txtAutor.Text = documentAct.autor;
            txtPublicacioEditorial.Text = documentAct.publicacioEditorial;
            txtPublicacioLloc.Text = documentAct.publicacioLloc;
            txtPublicacioEdicio.Text = documentAct.publicacioEdicio;
            txtPublicacioAny.Text = Convert.ToString(documentAct.publicacioAny);
            txtColeccioNom.Text = documentAct.coleccioNom;
            txtIsbn.Text = documentAct.isbn;
            txtDipositlegal.Text = documentAct.dipositLegal;
            cmbIdioma.Text = documentAct.idioma;
            cmbId_format.Text = documentAct.id_format;
            cmbId_materia.Text = documentAct.id_materia;
            cmbId_tema.Text = documentAct.id_tema;
            txtNumPagines.Text = Convert.ToString(documentAct.numPagines);
            chkActiu.Checked = documentAct.actiu;
            txtDirector.Text = documentAct.director;
            txtInterprets.Text = documentAct.interprets;
            txtCalificacio.Text = documentAct.calificacio;
            txtSinopsis.Text = documentAct.sinopsis;
            txtDuradaMinuts.Text = Convert.ToString(documentAct.duradaMinuts);
            cmbColor.Text = documentAct.color;

            txtImatge.Text = documentAct.imatge;
            txtNumUnitats.Text = Convert.ToString(documentAct.numUnitats);
            txtNumUnitatsDisponibles.Text = Convert.ToString(documentAct.numUnitatsDisponibles);
            txtDataAlta.Text = documentAct.dataAlta.ToString("dd/MM/yyyy");

            if (String.IsNullOrEmpty(documentAct.imatge) == false)
            {
                mostrarImatge();
            }


        }


        //*********************************************************************************************************+
        //Mou els camps desde pantalla ...
        private void moureCamps()
        {
            DateTime w_data = new DateTime();
            
            //Numeros buits
            if (txtPublicacioAny.Text.Trim()  == "") txtPublicacioAny.Text  = "0";
            if (txtNumPagines.Text.Trim() == "") txtNumPagines.Text = "0";
            if (txtNumUnitats.Text.Trim() == "") txtNumUnitats.Text = "0";
            if (txtNumUnitatsDisponibles.Text.Trim() == "") txtNumUnitatsDisponibles.Text = "0";
            if (txtDuradaMinuts.Text.Trim() == "") txtDuradaMinuts.Text = "0";

            if (txtSignatura.Text.Trim() == "") txtSignatura.Text = generarSignatura();

            documentAct.titol = txtTitol.Text;
            documentAct.signatura = txtSignatura.Text;
            documentAct.autor = txtAutor.Text;
            documentAct.publicacioEditorial = txtPublicacioEditorial.Text;
            documentAct.publicacioLloc = txtPublicacioLloc.Text;
            documentAct.publicacioEdicio = txtPublicacioEdicio.Text;
            documentAct.publicacioAny = Convert.ToInt32(txtPublicacioAny.Text);
            documentAct.coleccioNom = txtColeccioNom.Text;
            documentAct.isbn = txtIsbn.Text;
            documentAct.dipositLegal = txtDipositlegal.Text;
            documentAct.idioma = cmbIdioma.Text;
            documentAct.id_format = cmbId_format.Text;
            documentAct.id_materia = cmbId_materia.Text;
            documentAct.id_tema = cmbId_tema.Text;
            documentAct.numPagines = Convert.ToInt32(txtNumPagines.Text);
            documentAct.actiu = chkActiu.Checked;

            documentAct.director = txtDirector.Text;
            documentAct.interprets = txtInterprets.Text;
            documentAct.calificacio = txtCalificacio.Text;
            documentAct.sinopsis = txtSinopsis.Text;
            documentAct.duradaMinuts = Convert.ToInt32(txtDuradaMinuts.Text);
            documentAct.color = cmbColor.Text;

            documentAct.imatge = txtImatge.Text;
            documentAct.numUnitats = Convert.ToInt32(txtNumUnitats.Text);
            documentAct.numUnitatsDisponibles = Convert.ToInt32(txtNumUnitatsDisponibles.Text);
            w_data = DateTime.Parse(this.txtDataAlta.Text);
            documentAct.dataAlta = w_data;
        }

        //*********************************************************************************************************+
        //Buscar un document a la BD...
        private void buscarDocument()
        {
            documentAct = clientWCF.buscarDocument(id_documentAct);
        }

        
        //*********************************************************************************************************+
        //Afegir nou document a la BD...
        private void afegirDocument()
        {
            clientWCF.afegirDocument(documentAct);
        }

        //*********************************************************************************************************+
        //Modificar document de la BD...
        private void modificarDocument()
        {
            clientWCF.modificarDocument(documentAct);
        }

        //*********************************************************************************************************+
        //Eliminar document de la BD...
        private void eliminarDocument()
        {
            clientWCF.eliminarDocument(id_documentAct);
        }


        //*********************************************************************************************************+
        //Carregar combo Formats
        private void carregarCmbFormats()
        {
            ListFormats = clientWCF.llistarFormats().ToList();

            //netejem la llista per si te dades
            cmbId_format.Items.Clear();
            for (int i = 0; i < ListFormats.Count(); i++)
            {
                cmbId_format.Items.Add(ListFormats.ElementAt(i).id_format);
            }
        }

        //*********************************************************************************************************+
        //Carregar combo materies
        private void carregarCmbMateries()
        {
            ListMateries = clientWCF.llistarMateries().ToList();

            //Passem la Llista a una Table temporal
            cmbId_materia.Items.Clear();
            DataTable dtTmp = new DataTable();
            dtTmp.Columns.Add("Id", typeof(string));
            dtTmp.Columns.Add("Descripcio", typeof(string));
            for (int i = 0; i < ListMateries.Count(); i++)
            {
                dtTmp.Rows.Add(ListMateries.ElementAt(i).id_materia, ListMateries.ElementAt(i).materia);
            }
            cmbId_materia.DataSource = dtTmp;
            cmbId_materia.DisplayMember = "Id";
            cmbId_materia.ValueMember = "Descripcio";
            cmbId_materia.SelectedIndex = -1;
            cmbId_materia.Text = "";
        }

        //*********************************************************************************************************+
        //Carregar combo temes
        private void carregarCmbTemes()
        {
            ListTemes = clientWCF.llistarTemes().ToList();
            
            //Passem la Llista a una Table temporal
            cmbId_tema.Items.Clear();
            DataTable dtTmp = new DataTable();
            dtTmp.Columns.Add("Id", typeof(string));
            dtTmp.Columns.Add("Descripcio", typeof(string));
            for (int i = 0; i < ListTemes.Count(); i++)
            {
                dtTmp.Rows.Add(ListTemes.ElementAt(i).id_tema, ListTemes.ElementAt(i).tema);
            }
            cmbId_tema.DataSource = dtTmp;
            cmbId_tema.DisplayMember = "Id";
            cmbId_tema.ValueMember = "Descripcio";
            cmbId_tema.SelectedIndex = -1;
            cmbId_tema.Text = "";
        }

        //*********************************************************************************************************+
        //Carregar combo Idiomes
        private void carregarCmbIdiomes()
        {
            ListIdiomes = clientWCF.llistarIdiomes().ToList();

            //Passem la Llista a una Table temporal
            DataTable dtTmp = new DataTable();
            dtTmp.Columns.Add("Id", typeof(string));
            dtTmp.Columns.Add("Descripcio", typeof(string));
            for (int i = 0; i < ListIdiomes.Count(); i++)
            {
                dtTmp.Rows.Add(ListIdiomes.ElementAt(i).id_idioma, ListIdiomes.ElementAt(i).idioma);
            }

            cmbIdioma.Items.Clear();
            cmbIdioma.DataSource = dtTmp;
            cmbIdioma.DisplayMember = "Id";
            cmbIdioma.ValueMember = "Descripcio";
            cmbIdioma.SelectedIndex = -1;
            cmbIdioma.Text = "";

        }


        //*********************************************************************************************************+
        //Valors per defecte al donar d'alta
        private void altaPerDefecte()
        {
            DateTime avui = DateTime.Today;
            documentAct.id_document = 0;
            documentAct.actiu = true;
            documentAct.numUnitats = 1;
            documentAct.numUnitatsDisponibles = 1;
            documentAct.dataAlta = avui;
        }

        //*********************************************************************************************************+
        //Generar la signatura
        private string generarSignatura()
        {
            
            string part1 = "";
            string part2 = "";
            string part3 = "";
            string signatura = "";

            if (String.IsNullOrEmpty(cmbId_tema.Text) == true ||
                String.IsNullOrEmpty(txtTitol.Text) == true ||
                String.IsNullOrEmpty(txtAutor.Text) == true)
            {
                DialogResult result = MessageBox.Show("Per assignar la signatura cal informar Títol, Autor i Materia", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return signatura;
            }
            
            
            
            
            part1 = cmbId_tema.Text; 
            part2 = " (" + txtTitol.Text.Substring(0, 3).Trim() + ") ";
            part3 = txtAutor.Text.Substring(0, 3);
            part1 = part1.ToUpper();
            part2 = part2.ToUpper();
            part3 = part3.ToLower();

            signatura = part1 + part2 + part3;

            //signatura = txtId_temaHelp.Text + " (" + txtTitol.Text.Substring(0,3) + ") " + txtAutor.Text.Substring(0,3);
            return signatura;
        }

        //*********************************************************************************************************+
        //Mostrar iamtge per pantalla
        private void mostrarImatge()
        {
            //picImatge.ImageLocation = "C:\\UOC\\TFC\\UOCBiblioteca\\WebSite\\ImagesDoc\\" + documentAct.imatge;
            picImatge.ImageLocation = pathPrograma + "\\WebSite\\ImagesDoc\\" + documentAct.imatge;
            lblPathImatge.Text = pathPrograma + "\\WebSite\\ImagesDoc";
            lblPathImatge.Text = lblPathImatge.Text.ToLower();
            lblPathImatge.Visible = true;
            lblUbicacio.Visible = true;
        }


        //*********************************************************************************************************+
        private void btnSignatura_Click(object sender, EventArgs e)
        {
            txtSignatura.Text = generarSignatura();
            txtSignatura.Focus();
        }

        //*********************************************************************************************************+
        // Quadre dialeg per escollir la imatge...
        private void btnAbrir_Click(object sender, EventArgs e)
        {
            string pathCorrecte = "";
            string arxiu = "";

            //Directorio d'imatges...
            pathCorrecte = pathPrograma + "\\WebSite\\ImagesDoc";
            pathCorrecte = pathCorrecte.ToUpper();
            openFileDialog1.InitialDirectory= pathCorrecte;


            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                arxiu = Path.GetFileName(openFileDialog1.FileName);

                if (Path.GetExtension(openFileDialog1.FileName).ToUpper() != ".JPG")
                {
                    DialogResult result = MessageBox.Show("L'arxiu d'imatge no és un JPG", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                if (pathCorrecte != Path.GetDirectoryName(openFileDialog1.FileName).ToUpper())
                {
                    DialogResult result = MessageBox.Show("L'arxiu d'imatge ha d'estar necessàriament al directori" + 
                                                            System.Environment.NewLine + System.Environment.NewLine +
                                                            pathCorrecte + 
                                                            System.Environment.NewLine , "Document", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return;
                }

                //Mostrar foto...
                txtImatge.Text = arxiu;
                documentAct.imatge = arxiu;
                mostrarImatge();
            }
        }



        //*********************************************************************************************************+
        private void txtNumPagines_KeyPress(object sender, KeyPressEventArgs e)
        {
            //e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }


        //*********************************************************************************************************+
        // Buscar el path desde a on s'executa el programa
        //      Ex:   C:\UOC\TFC\UOCBIBLIOTECA
        private void buscarPathPrograma()
        {
            string path = "";
            int i=0;

            path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
            path = path.ToUpper();

            if (path.Substring(0, 4) == "FILE")
            {
                path = path.Substring(6, path.Length - 6);
                i = path.LastIndexOf("\\UOCBIBLIOTECA");
                pathPrograma = path.Substring(0, i+1) + "UOCBIBLIOTECA";
            }
        }


        //*********************************************************************************************************+
        //Validar data
        private Boolean isDate(string data)
        {
            DateTime dateparse;
            if (DateTime.TryParseExact(data, "dd/MM/yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out dateparse)) return true;
            return false;
        }

        //*********************************************************************************************************+
        //Validar numero
        private Boolean isNumeric(string numero)
        {
            int number = 0;
            if (int.TryParse(numero.Trim(), out number)) return true;
            return false;
        }



    }//fi class
}//fi namespace
