﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using Negoci;
using Terminal.ServiceReference1;
using NegociEntitats;

namespace Terminal
{
    public partial class frm3Soci : Form
    {
        private ServiceReference1.Service1Client clientWCF = new ServiceReference1.Service1Client(); // Instancia del servei WCF
        //private simulawcfCN clientWCF = new simulawcfCN(); // simula instancia WCF

        private List<tipusSocisEntity> ListTipusSocis;
        private List<prestecsEntity> ListPrestecs; 

        private socisEntity sociAct = new socisEntity();
        private int id_sociAct = 0;
         
        public frm3Soci(int id_soci)
        {
            InitializeComponent();
            id_sociAct = id_soci;
        }

        private void frm3Soci_Load(object sender, EventArgs e)
        {
            //Carregar combos...
            carregarCmbSexe();
            carregarCmbTipusSocis();

            if (id_sociAct == 0)
            {
                altaPerDefecte();
            }
            else
            {
                buscarSoci();
                buscarPrestecs();
            }

            moureCamps_a_Pantalla();

            //Desactivar opcions segons usuari actiu
            //  si rol=2 (usuari normal), desactivar opcions
            gblParametres parametres = gblParametres.Instance();
            if (parametres.usuariRol == 2)
            {
                acceptarButton.Enabled = false;
                eliminarButton.Enabled = false;
            }

        
        }

        private void acceptarButton_Click(object sender, EventArgs e)
        {
            //Valida que els camps estiguin Ok...
            Boolean correcte = true;
            correcte = validarCamps();
            if (!correcte)
            {
                return;
            }

            if (id_sociAct == 0)
            {
                moureCamps();
                afegirSoci();
                this.Dispose();
            }
            else
            {
                moureCamps();
                modificarSoci();
                this.Dispose();
            }
        }

        //********************************************************************************************
        private void eliminarButton_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("Confirmar l'eliminació", "Eliminar Soci", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

            if (result == DialogResult.OK)
            {
                eliminarSoci();
                this.Dispose();
            }
        }

        private void cancelarButton_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }


        //*********************************************************************************************************+
        //Validar els camps introduits...
        private Boolean validarCamps()
        {
            Boolean correcte = true;

            //Validar nom... 
            if (String.IsNullOrEmpty(txtNom.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar NOM", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtNom.Focus();
                return false;
            }

            //Validar cognom...
            if (String.IsNullOrEmpty(txtCognoms.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar COGNOMS", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtCognoms.Focus();
                return false;
            }

            //Validar tipus Soci...
            if (String.IsNullOrEmpty(cmbId_tipusSoci.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar TIPUS SOCI", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                cmbId_tipusSoci.Focus();
                return false;
            }

            //Validar DNI...
            if (String.IsNullOrEmpty(txtDni.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar DNI", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtDni.Focus();
                return false;
            }

            //Validar Adreça...
            if (String.IsNullOrEmpty(txtAdreca.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar ADREÇA", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtAdreca.Focus();
                return false;
            }

            //Validar CP...
            if (String.IsNullOrEmpty(txtCp.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar CODI POSTAL", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtCp.Focus();
                return false;
            }


            //Validar Poblacacio...            
            if (String.IsNullOrEmpty(txtPoblacio.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar POBLACIO", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtPoblacio.Focus();
                return false;
            }


            //Validar data naixement...
            if (String.IsNullOrEmpty(txtDataNaixement.Text) == true)
            {
                DialogResult result = MessageBox.Show("Falta informar DATA NAIXEMENT", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtDataNaixement.Focus();
                return false;
            }
            if (String.IsNullOrEmpty(txtDataNaixement.Text) == false)
            {
                if (this.isDate(txtDataNaixement.Text) == false)
                {
                    DialogResult result = MessageBox.Show("DATA NAIXEMENT Incorrecte" + System.Environment.NewLine + System.Environment.NewLine + "Format: dd/mm/aaaa", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDataNaixement.Focus();
                    return false;
                }

            }

            //Validar dataAlta...
            if (String.IsNullOrEmpty(txtDataAlta.Text) == false)
            {
                if (this.isDate(txtDataAlta.Text) == false)
                {
                    DialogResult result = MessageBox.Show("DATA ALTA Incorrecte" + System.Environment.NewLine + System.Environment.NewLine + "Format: dd/mm/aaaa", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDataAlta.Focus();
                    return false;
                }
            }

            //Validar dataBaixa...
            if (String.IsNullOrEmpty(txtDataBaixa.Text) == false)
            {
                if (this.isDate(txtDataBaixa.Text) == false)
                {
                    DialogResult result = MessageBox.Show("DATA BAIXA Incorrecte" + System.Environment.NewLine + System.Environment.NewLine + "Format: dd/mm/aaaa", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDataBaixa.Focus();
                    return false;
                }
            }

            //Validar email...
            if (String.IsNullOrEmpty(txtEmail.Text) == false)
            {
                txtEmail.Text = txtEmail.Text.Trim();

                if (txtEmail.Text.IndexOf("@") < 1 || txtEmail.Text.IndexOf(".") < 3 || txtEmail.Text.Length < 5)
                {
                    DialogResult result = MessageBox.Show("EMAIL no te un format correcte  xxxxx@xxxxx.xxx" , "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtEmail.Text = sociAct.email;
                    txtEmail.Focus();
                    return false;
                }

                if (clientWCF.existeixEmailSoci(sociAct.id_soci, txtEmail.Text) == true)
                {
                    DialogResult result = MessageBox.Show("EMAIL ja existent en el sistema" + System.Environment.NewLine + System.Environment.NewLine + "No es pot registrar aquest valor", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtEmail.Text = sociAct.email;
                    txtEmail.Focus();
                    return false;
                }
            }

            //Validar dni...
            if (String.IsNullOrEmpty(txtDni.Text) == false)
            {
                txtDni.Text = txtDni.Text.Trim();
                if (clientWCF.existeixDniSoci(sociAct.id_soci, txtDni.Text) == true)
                {
                    DialogResult result = MessageBox.Show("DNI ja existent en el sistema" + System.Environment.NewLine + System.Environment.NewLine + "No es pot registrar aquest valor", "Soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtDni.Text = sociAct.dni;
                    txtDni.Focus();
                    return false;
                }
            }
            //Retorn
            return correcte;
        }

        //*********************************************************************************************************+
        //Mou els camps a la pantalla ...
        private void moureCamps_a_Pantalla()
        {
            txtId_soci.Text = Convert.ToString(sociAct.id_soci);
            if (sociAct.id_soci == 0) txtId_soci.Text = "*** nou ***";
            txtNom.Text = sociAct.nom;
            txtCognoms.Text = sociAct.cognoms;
            cmbSexe.Text = sociAct.sexe;
            txtDni.Text = sociAct.dni;
            txtDataNaixement.Text = sociAct.dataNaixement.ToString("dd/MM/yyyy");
            if (txtDataNaixement.Text == "01/01/0001" || txtDataNaixement.Text == "01/01/1753")
            {
                txtDataNaixement.Text = "";
            }
            cmbId_tipusSoci.Text = Convert.ToString(sociAct.id_tipusSoci);
            txtAdreca.Text = sociAct.adreca;
            txtCp.Text = sociAct.cp;
            txtPoblacio.Text = sociAct.poblacio;
            txtProvincia.Text = sociAct.provincia;
            txtTelefon.Text = sociAct.telefon;
            txtEmail.Text = sociAct.email;
            txtContrasenya.Text = sociAct.contrasenya;
            txtDataAlta.Text = sociAct.dataAlta.ToString("dd/MM/yyyy");
            if (txtDataAlta.Text == "01/01/0001" || txtDataAlta.Text == "01/01/1753")
            {
                txtDataAlta.Text = "";
            }
            txtDataBaixa.Text = sociAct.dataBaixa.ToString("dd/MM/yyyy");
            if (txtDataBaixa.Text == "01/01/0001" || txtDataBaixa.Text == "01/01/1753" )
            {
                txtDataBaixa.Text = "";
            }
            chkActiu.Checked = sociAct.actiu;
            chkEsBaixa.Checked = sociAct.esBaixa;
        }


        //*********************************************************************************************************+
        //Mou els camps desde pantalla ...
        private void moureCamps()
        {
            DateTime w_data = new DateTime();
            String x_data = "";
            
            //Si no hi ha contrasenya, la calcula
            if (String.IsNullOrEmpty(txtContrasenya.Text) == true)
            {
                w_data = DateTime.Parse(this.txtDataNaixement.Text);
                x_data = w_data.ToString("dd/MM/yyyy");
                txtContrasenya.Text = x_data.Substring(0, 2) + x_data.Substring(3, 2) + x_data.Substring(6, 4);
            }
            
            sociAct.nom = txtNom.Text;
            sociAct.cognoms=txtCognoms.Text;
            sociAct.sexe = cmbSexe.Text;
            sociAct.dni = txtDni.Text;
            w_data = DateTime.Parse(this.txtDataNaixement.Text);
            sociAct.dataNaixement = w_data;
            sociAct.id_tipusSoci = Convert.ToInt32(cmbId_tipusSoci.Text);
            sociAct.adreca = txtAdreca.Text;
            sociAct.cp = txtCp.Text;
            sociAct.poblacio = txtPoblacio.Text;
            sociAct.provincia = txtProvincia.Text;
            sociAct.telefon = txtTelefon.Text;
            sociAct.email = txtEmail.Text;
            sociAct.contrasenya = txtContrasenya.Text;
            w_data = DateTime.Parse(this.txtDataAlta.Text);
            sociAct.dataAlta = w_data;
            if (String.IsNullOrEmpty(txtDataBaixa.Text) == false)
            {
                w_data = DateTime.Parse(this.txtDataBaixa.Text);
            }
            sociAct.actiu = chkActiu.Checked;
            sociAct.esBaixa = chkEsBaixa.Checked;

        }

        //*********************************************************************************************************+
        //Buscar un soci a la BD...
        private void buscarSoci()
        {
            sociAct = clientWCF.buscarSoci(id_sociAct);

        }


        //*********************************************************************************************************+
        //Buscar els prestecs d'un soci a la BD...
        private void buscarPrestecs()
        {
            ListPrestecs = clientWCF.llistarPrestecsIdSoci(id_sociAct).ToList();
            carregarGrid();
        }


        
        //*********************************************************************************************************+
        //Afegir nou soci a la BD...
        private void afegirSoci()
        {
            clientWCF.afegirSoci(sociAct);
        }

        //*********************************************************************************************************+
        //Modificar soci de la BD...
        private void modificarSoci()
        {
            clientWCF.modificarSoci(sociAct);
        }

        //*********************************************************************************************************+
        //Eliminar soci de la BD...
        private void eliminarSoci()
        {
            clientWCF.eliminarSoci(id_sociAct);
        }


        //*********************************************************************************************************+
        //Carregar combo sexe
        private void carregarCmbSexe()
        {
            //Creem taula temporal pels valors del sexe
            cmbSexe.Items.Clear();
            DataTable dtTmp = new DataTable();
            dtTmp.Columns.Add("Id", typeof(string));
            dtTmp.Columns.Add("Descripcio", typeof(string));
            dtTmp.Rows.Add("H", "Home");
            dtTmp.Rows.Add("D", "Dona");
            cmbSexe.DataSource = dtTmp;
            cmbSexe.DisplayMember = "Id";
            cmbSexe.ValueMember = "Descripcio";
            cmbSexe.SelectedIndex = -1;
            cmbSexe.Text = "";

        }

        //*********************************************************************************************************+
        //Carregar combo tipusSocis
        private void carregarCmbTipusSocis()
        {
            ListTipusSocis = clientWCF.llistarTipusSocis().ToList();

            //Passem la Llista a una Table temporal
            cmbId_tipusSoci.Items.Clear();
            DataTable dtTmp = new DataTable();
            dtTmp.Columns.Add("Id", typeof(string));
            dtTmp.Columns.Add("Descripcio", typeof(string));
            for (int i = 0; i < ListTipusSocis.Count(); i++)
            {
                dtTmp.Rows.Add(ListTipusSocis.ElementAt(i).id_tipusSoci, ListTipusSocis.ElementAt(i).tipusSoci);
            }
            cmbId_tipusSoci.DataSource = dtTmp;
            cmbId_tipusSoci.DisplayMember = "Id";
            cmbId_tipusSoci.ValueMember = "Descripcio";
            cmbId_tipusSoci.SelectedIndex = -1;
            cmbId_tipusSoci.Text = "";
        }

        //-------------------------------------------------------------------------------------------------
        // Carrega la llista al datagridView .....
        private void carregarGrid()
        {
            if (ListPrestecs == null) return;
            if (ListPrestecs.Count == 0) return;

            //Passar la llista de documents obtinguda al grid...
            for (int i = 0; i < ListPrestecs.Count(); i++)
            {
                gridPrestecs.Rows.Add();
                gridPrestecs.Rows[i].Cells[0].Value = ListPrestecs.ElementAt(i).id_prestec;
                gridPrestecs.Rows[i].Cells[1].Value = ListPrestecs.ElementAt(i).id_document;
                gridPrestecs.Rows[i].Cells[2].Value = ListPrestecs.ElementAt(i).signatura;
                gridPrestecs.Rows[i].Cells[3].Value = ListPrestecs.ElementAt(i).titol;
                gridPrestecs.Rows[i].Cells[4].Value = ListPrestecs.ElementAt(i).dataIniciPrestec.ToString("dd/MM/yyyy");
                gridPrestecs.Rows[i].Cells[5].Value = ListPrestecs.ElementAt(i).dataFiPrestec.ToString("dd/MM/yyyy");
                gridPrestecs.Rows[i].Cells[6].Value = ListPrestecs.ElementAt(i).dataRetornPrestec.ToString("dd/MM/yyyy");
                if (ListPrestecs.ElementAt(i).dataRetornPrestec.ToString("dd/MM/yyyy") == "01/01/0001")
                {
                    gridPrestecs.Rows[i].Cells[6].Value = " ";
                }
                if (ListPrestecs.ElementAt(i).retornat == false)
                {
                    gridPrestecs.Rows[i].Cells[6].Value = " ";
                }

            }

            gridPrestecs.Refresh();

        }


        //*********************************************************************************************************+
        //Valors per defecte al donar d'alta
        private void altaPerDefecte()
        {
            DateTime avui = DateTime.Today;
            sociAct.id_soci = 0;
            sociAct.actiu = true;
            sociAct.esBaixa = false;
            sociAct.dataAlta = avui;
        }

        //*********************************************************************************************************+
        //Validar data
        private Boolean isDate(string data)
        {
            DateTime dateparse;
            if (DateTime.TryParseExact(data, "dd/MM/yyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out dateparse)) return true;
            return false;
        }

        //*********************************************************************************************************+
        //Validar numero
        private Boolean isNumeric(string numero)
        {
            int number = 0;
            if (int.TryParse(numero.Trim(), out number)) return true;
            return false;
        }

        private void tabPage1_Click(object sender, EventArgs e)
        {

        }




    }//fi class
}//fi namespace
