﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using Negoci;
using Terminal.ServiceReference1;
using NegociEntitats;

namespace Terminal
{
    public partial class frm5TipusSoci : Form
    {
        private ServiceReference1.Service1Client clientWCF = new ServiceReference1.Service1Client(); // Instancia del servei WCF
        //private simulawcfCN clientWCF = new simulawcfCN(); // simula WCF

        private tipusSocisEntity tipusSociAct = new tipusSocisEntity();
        private int id_tipusSociAct = 0;
        private Boolean swnouregistre = false;

        //********************************************************************************************
        public frm5TipusSoci(int id_tipusSoci)
        {
            InitializeComponent();
            id_tipusSociAct = id_tipusSoci; 
        }

        //********************************************************************************************
        private void frm5TipusSoci_Load(object sender, EventArgs e)
        {
            if (id_tipusSociAct == 0)
            {
                altaPerDefecte();
                swnouregistre = true;
                txtId_tipusSoci.Focus();
            }
            else
            {
                buscarTipusSoci();
                txtId_tipusSoci.ReadOnly = true;
            }
            moureCamps_a_Pantalla();

        }


        //********************************************************************************************
        private void acceptarButton_Click(object sender, EventArgs e)
        {
            //Valida que els camps estiguin Ok...
            Boolean correcte = true;
            correcte = validarCamps();
            if (!correcte)
            {
                return;
            }

            if (swnouregistre)
            {
                moureCamps();
                afegirTipusSoci();
                this.Dispose();
            }
            else
            {
                moureCamps();
                modificarTipusSoci();
                this.Dispose();
            }
        }

        //********************************************************************************************
        private void eliminarButton_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("Confirmar l'eliminació", "Eliminar tipus soci", MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

            if (result == DialogResult.OK)
            {
                eliminarTipusSoci();
                this.Dispose();
            }
        }

        private void cancelarButton_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }


        //*********************************************************************************************************+
        //Validar els camps introduits...
        private Boolean validarCamps()
        {
            Boolean correcte = true;

            //Validar id
            if (String.IsNullOrEmpty(txtId_tipusSoci.Text) == true)
            {
                DialogResult result = MessageBox.Show("Faltar informar ID TIPUS SOCI", "Tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtId_tipusSoci.Focus();
                return false;
            }
            if (String.IsNullOrEmpty(txtId_tipusSoci.Text) == false)
            {
                if (this.isNumeric(txtId_tipusSoci.Text) == false)
                {
                    DialogResult result = MessageBox.Show("ID TIPUS SOCI sols admet números", "Document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtId_tipusSoci.Focus();
                    return false;
                }
                else
                {
                    if (Convert.ToInt32(txtId_tipusSoci.Text) > 10)
                    {
                        DialogResult result = MessageBox.Show("El ID TIPUS SOCI ha de ser menor a 10", "Tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        txtId_tipusSoci.Focus();
                        return false;
                    }
                }
            }

            //Validar descripcio 
            if (String.IsNullOrEmpty(txtTipusSoci.Text) == true)
            {
                DialogResult result = MessageBox.Show("Faltar informar TIPUS SOCI", "Tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtTipusSoci.Focus();
                return false;
            }

            //Validar duplicitat...
            if (swnouregistre)
            {
                if (clientWCF.existeixTipusSoci(Convert.ToInt32(txtId_tipusSoci.Text)))
                {
                    DialogResult result = MessageBox.Show("ID TIPUS SOCI duplicat", "Tipus soci", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    txtId_tipusSoci.Focus();
                    return false;
                }
            }


            //Retorn
            return correcte;
        }

        //*********************************************************************************************************+
        //Mou els camps a la pantalla ...
        private void moureCamps_a_Pantalla()
        {
            txtId_tipusSoci.Text = tipusSociAct.id_tipusSoci.ToString();
            txtTipusSoci.Text = tipusSociAct.tipusSoci;
        }


        //*********************************************************************************************************+
        //Mou els camps desde pantalla ...
        private void moureCamps()
        {
            tipusSociAct.id_tipusSoci = Convert.ToInt32(txtId_tipusSoci.Text);
            tipusSociAct.tipusSoci = txtTipusSoci.Text;
        }

        //*********************************************************************************************************+
        //Buscar un registre a la BD...
        private void buscarTipusSoci()
        {
            tipusSociAct = clientWCF.buscarTipusSoci(id_tipusSociAct); 

        }


        //*********************************************************************************************************+
        //Afegir nou registre a la BD...
        private void afegirTipusSoci()
        {
            clientWCF.afegirTipusSoci(tipusSociAct);
        }

        //*********************************************************************************************************+
        //Modificar registre de la BD...
        private void modificarTipusSoci()
        {
            clientWCF.modificarTipusSoci(tipusSociAct);
        }

        //*********************************************************************************************************+
        //Eliminar registre de la BD...
        private void eliminarTipusSoci()
        {
            clientWCF.eliminarTipusSoci(id_tipusSociAct);
        }




        //*********************************************************************************************************+
        //Valors per defecte al donar d'alta
        private void altaPerDefecte()
        {
            lblnou.Visible = true;
        }


        //*********************************************************************************************************+
        //Validar numero
        private Boolean isNumeric(string numero)
        {
            int number = 0;
            if (int.TryParse(numero.Trim(), out number)) return true;
            return false;
        }




    }//fi class
}//fi namespace
